//**********************************************************************;
// Copyright (c) 2015, Intel Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.
//**********************************************************************;

#include "sapi/tpm20.h"
#include "sysapi_util.h"
#include "tss2_endian.h"

TSS2_RC Tss2_Sys_GetDecryptParam(
    TSS2_SYS_CONTEXT *sysContext,
    size_t *decryptParamSize,
    const uint8_t **decryptParamBuffer)
{
    TPM2B *decryptParam;

    if (!decryptParamSize || !decryptParamBuffer || !sysContext)
        return TSS2_SYS_RC_BAD_REFERENCE;

    if (SYS_CONTEXT->previousStage != CMD_STAGE_PREPARE)
        return TSS2_SYS_RC_BAD_SEQUENCE;

    if (SYS_CONTEXT->decryptAllowed == 0)
        return TSS2_SYS_RC_NO_DECRYPT_PARAM;

    /* Get first parameter and return its size and a pointer to it. */
    decryptParam = (TPM2B *)(SYS_CONTEXT->cpBuffer);
    *decryptParamSize = BE_TO_HOST_16(decryptParam->size);
    *decryptParamBuffer = decryptParam->buffer;

    return TSS2_RC_SUCCESS;
}

TSS2_RC Tss2_Sys_SetDecryptParam(
    TSS2_SYS_CONTEXT *sysContext,
    size_t decryptParamSize,
    const uint8_t *decryptParamBuffer)
{
    size_t currDecryptParamSize;
    const uint8_t *currDecryptParamBuffer;
    TSS2_RC rval;
    UINT32 currCommandSize;
    const UINT8 *src, *limit;
    UINT8 *dst;
    UINT32 len;

    if (!decryptParamBuffer || !sysContext)
        return TSS2_SYS_RC_BAD_REFERENCE;

    if (BE_TO_HOST_32(SYS_REQ_HEADER->commandSize) +
        decryptParamSize > SYS_CONTEXT->maxCmdSize)
        return TSS2_SYS_RC_INSUFFICIENT_CONTEXT;

    rval = Tss2_Sys_GetDecryptParam(sysContext, &currDecryptParamSize,
                                    &currDecryptParamBuffer);
    if (rval)
        return rval;

    if (currDecryptParamSize == 0 && SYS_CONTEXT->decryptNull)
    {
        if (decryptParamSize < 1)
            return TSS2_SYS_RC_BAD_VALUE;

        /* Move stuff around. First move current cpBuffer down. */
        src = SYS_CONTEXT->cpBuffer + 2;
        dst = SYS_CONTEXT->cpBuffer + SYS_CONTEXT->cpBufferUsedSize + 2;
        len = SYS_CONTEXT->cpBufferUsedSize - 2;
        limit = SYS_CONTEXT->cmdBuffer + SYS_CONTEXT->maxCmdSize;

        if (dst + len > limit)
            return TSS2_SYS_RC_INSUFFICIENT_CONTEXT;

        memmove(dst, src, len);

        SYS_CONTEXT->cpBufferUsedSize += decryptParamSize;
        *(UINT16 *)SYS_CONTEXT->cpBuffer = HOST_TO_BE_16(decryptParamSize);

        src = decryptParamBuffer;
        dst = (UINT8 *) currDecryptParamBuffer;
        len = decryptParamSize;
        limit = SYS_CONTEXT->cmdBuffer + SYS_CONTEXT->maxCmdSize;

        if (dst + len > limit)
            return TSS2_SYS_RC_INSUFFICIENT_CONTEXT;

        /* Now copy in the encrypted decrypt param. */
        memmove(dst, src, len);

        /* And fixup the command size. */
        currCommandSize = BE_TO_HOST_32(SYS_REQ_HEADER->commandSize);
        currCommandSize += decryptParamSize;
        SYS_REQ_HEADER->commandSize = HOST_TO_BE_32(currCommandSize);
    }
    else
    {
        if (decryptParamSize != currDecryptParamSize)
            return TSS2_SYS_RC_BAD_SIZE;

        *(UINT16 *)SYS_CONTEXT->cpBuffer = HOST_TO_BE_16(decryptParamSize);

        src = decryptParamBuffer;
        dst = (UINT8 *) currDecryptParamBuffer;
        len = decryptParamSize;
        limit = SYS_CONTEXT->cmdBuffer + SYS_CONTEXT->maxCmdSize;

        if (dst + len > limit)
            return TSS2_SYS_RC_INSUFFICIENT_CONTEXT;

        memmove(dst, src, len);
    }

    return rval;
}
